@@ -1,7 +1,7 @@
 package Chemistry::Atom;
 
-$VERSION = '0.26';
-# $Id: Atom.pm,v 1.27 2004/08/06 23:43:38 itubert Exp $
+$VERSION = '0.35';
+# $Id: Atom.pm,v 1.41 2005/05/20 19:01:04 itubert Exp $
 
 =head1 NAME
 
@@ -12,9 +12,9 @@ Chemistry::Atom - Chemical atoms as objects in molecules
     use Chemistry::Atom;
 
     my $atom = new Chemistry::Atom(
-	id => 'a1',
-	coords => [$x, $y, $z],
-	symbol => 'Br'
+        id => 'a1',
+        coords => [$x, $y, $z],
+        symbol => 'Br'
     );
 
     print $atom->print;
@@ -53,6 +53,7 @@ use Math::VectorReal qw(O vector);
 use Math::Trig;
 use Carp;
 use base qw(Chemistry::Obj Exporter);
+use List::Util qw(first);
 
 our @EXPORT_OK = qw(distance angle dihedral angle_deg dihedral_deg);
 our %EXPORT_TAGS = (
@@ -180,7 +181,7 @@ sub symbol {
     my $self = shift;
 
     if(@_) {
-	$_[0] =~ s/ //g;
+        $_[0] =~ s/ //g;
         $self->{Z} = $ELEMENTS{$_[0]};
         $self->{symbol} = $_[0];
         return $self;
@@ -191,27 +192,53 @@ sub symbol {
 
 =item $atom->mass($new_mass)
 
-Sets and returns the atomic mass in atomic mass units. By default, relative
-atomic masses from the 1995 IUPAC recommendation are used. (Table stolen from
-the Chemistry::MolecularMass module by Maksim A. Khrapov).
+Sets and returns the atomic mass in atomic mass units. Any arbitrary mass may
+be set explicitly by using this method. However, if no mass is set explicitly
+and this method is called as an accessor, the return value is the following:
+
+1) If the mass number is undefined (see the mass_number method below), the
+relative atomic mass from the 1995 IUPAC recommendation is used. (Table stolen
+from the Chemistry::MolecularMass module by Maksim A.  Khrapov).
+
+2) If the mass number is defined and the L<Chemistry::Isotope> module is 
+available and it knows the mass for the isotope, the exact mass of the isotope
+is used; otherwise, the mass number is returned.
 
 =cut
 
 sub mass {
-    my ($self, $mass) = @_;
-    if(defined $mass) {
-        $self->{mass} = $mass;
+    my $self = shift;
+    if (@_) {
+        $self->{mass} = shift;
         return $self;
     } else {
-        if (exists $self->{mass}) {
-            $mass = $self->{mass};
+        if (defined $self->{mass}) {
+            return $self->{mass};
+        } elsif (defined $self->{mass_number}) {
+            if (eval { require Chemistry::Isotope } and 
+                my $m = Chemistry::Isotope::isotope_mass(
+                    $self->{mass_number}, $self->{Z})
+            ) {
+                return $m;
+            } else {
+                return $self->{mass_number};
+            }
         } else {
-            $mass = $Atomic_masses{$self->symbol};
+            return $Atomic_masses{$self->symbol};
         }
     }
-    $mass;
 }
 
+=item $atom->mass_number($new_mass_number)
+
+Sets or gets the mass number. The mass number is undefined unless is 
+set explicitly (this module does not try to guess a default mass number based
+on the natural occuring isotope distribution).
+
+=cut
+
+Chemistry::Obj::accessor('mass_number');
+
 =item $atom->coords
 
     my $vector = $atom->coords;  # get a Math::VectorReal object
@@ -244,16 +271,16 @@ sub coords {
 =item $atom->internal_coords
 
     # get a Chemistry::InternalCoords object
-    my $ic = $atom->coords;      
+    my $ic = $atom->internal_coords;      
 
     # set a Chemistry::InternalCoords object 
-    $atom->coords($vic);         
+    $atom->internal_coords($vic);         
 
     # also accepts array refs 
-    $atom->coords([8, 1.54, 7, 109.47, 6, 120.0]);   
+    $atom->internal_coords([8, 1.54, 7, 109.47, 6, 120.0]);   
 
     # also accepts lists
-    $atom->coords(8, 1.54, 7, 109.47, 6, 120.0);    
+    $atom->internal_coords(8, 1.54, 7, 109.47, 6, 120.0);    
 
 Sets or gets the atom's internal coordinates. It can take as a parameter a
 Chemistry::InternalCoords object, a reference to an array, or the list of
@@ -287,7 +314,7 @@ sub internal_coords {
 =item $atom->x3, $atom->y3, $atom->z3
 
 Get the x, y or z 3D coordinate of the atom. This methods are just accessors
-that don't change the coordinats. $atom->x3 is short for 
+that don't change the coordinates. $atom->x3 is short for 
 ($atom->coords->array)[0], and so on.
 
 =cut
@@ -302,15 +329,15 @@ Set or get the formal charge of the atom.
 
 =cut
 
-sub formal_charge {
-    my $self = shift;
-    if (@_) {
-        ($self->{formal_charge}) = @_;
-        return $self;
-    } else {
-        return $self->{formal_charge};
-    }
-}
+Chemistry::Obj::accessor('formal_charge');
+
+=item $atom->formal_radical($radical)
+
+Set or get the formal radical multiplicity for the atom.
+
+=cut
+
+Chemistry::Obj::accessor('formal_radical');
 
 =item $atom->implicit_hydrogens($h_count)
 
@@ -318,10 +345,7 @@ Set or get the number of implicit hydrogen atoms bonded to the atom.
 
 =cut
 
-sub implicit_hydrogens {
-    my ($self) = shift;
-    $self->hydrogens(@_);
-}
+sub implicit_hydrogens { shift->hydrogens(@_) }
 
 =item $atom->hydrogens($h_count)
 
@@ -330,19 +354,12 @@ Set or get the number of implicit hydrogen atoms bonded to the atom
 
 =cut
 
-sub hydrogens {
-    my $self = shift;
-    if (@_) {
-        ($self->{hydrogens}) = @_;
-        return $self;
-    } else {
-        return $self->{hydrogens};
-    }
-}
+Chemistry::Obj::accessor('hydrogens');
 
 =item $atom->total_hydrogens($h_count)
 
-Set or get the total number of hydrogen atoms bonded to the atom.
+Get the total number of hydrogen atoms bonded to the atom (implicit +
+explicit).
 
 =cut
 
@@ -352,6 +369,107 @@ sub total_hydrogens {
     $self->hydrogens + grep { $_->symbol eq 'H' } $self->neighbors;
 }
 
+=item $atom->sprout_hydrogens
+
+Convert all the implicit hydrogens for this atom to explicit hydrogens. Note:
+it does B<not> generate coordinates for the new atoms.
+
+=cut
+
+sub sprout_hydrogens {
+    my ($self) = @_;
+    for (1 .. $self->implicit_hydrogens || 0) {
+        $self->parent->new_bond(
+            atoms => [$self, $self->parent->new_atom(symbol => 'H')]);
+    }
+    $self->implicit_hydrogens(0);
+}
+
+=item $atom->collapse_hydrogens
+
+Delete neighboring hydrogen atoms and add them as implicit hydrogens for this
+atom.
+
+=cut
+
+sub collapse_hydrogens {
+    my ($self) = @_;
+    no warnings 'uninitialized';
+    my $implicit = $self->implicit_hydrogens;
+    for my $nei ($self->neighbors) {
+        $nei->delete, $implicit++ if $nei->symbol eq 'H';
+    }
+    $self->implicit_hydrogens($implicit);
+}
+
+my %VALENCE_TABLE = (
+    Br => 1, Cl => 1, B => 3, C => 4, N => 3, O => 2, P => 3, S => 2, 
+    F => 1, I => 1,
+);
+
+# to make it easier to test
+sub _calc_implicit_hydrogens {
+    my ($self, $symbol, $valence, $charge, $radical) = @_;
+    no warnings 'uninitialized';
+
+    my $h_count = $VALENCE_TABLE{$symbol} - $valence;
+    # should account for non-kekulized aromatic bonds
+
+    # some common charge situations
+    if (($symbol =~ /^[NOS]$/) && $charge == -1) {
+        $h_count--;
+    } elsif ($symbol =~ /^[NOSP]$/ && $charge == 1) {
+        $h_count++;
+    } elsif ($symbol eq 'C' && $charge) {
+        $h_count--;
+    } elsif ($symbol eq 'B' && $charge == -1) {
+        $h_count++;
+    }
+
+    # some common radical situations
+    if ($radical == 1 or $radical == 3) {
+        $h_count -=2;
+    } elsif ($radical == 2) {
+        $h_count--;
+    }
+
+    $h_count = 0 if $h_count < 0;
+    $h_count;
+}
+
+=item $atom->calc_implicit_hydrogens
+
+Use heuristics to figure out how many implicit hydrogens should the atom have
+to satisfy its normal "organic" valence. Returns the number of hydrogens but
+does not affect the atom object.
+
+=cut
+
+sub calc_implicit_hydrogens {
+    my ($self) = @_;
+    $self->_calc_implicit_hydrogens(
+        $self->symbol, $self->explicit_valence, 
+        $self->formal_charge, $self->formal_radical,
+    );
+}
+
+=item $atom->add_implicit_hydrogens
+
+Similar to calc_implicit_hydrogens, but it also sets the number of implicit
+hydrogens in the atom to the new calculated value. Equivalent to
+
+    $atom->implicit_hydrogens($atom->calc_implicit_hydrogens);
+
+It returns the atom object.
+
+=cut
+
+sub add_implicit_hydrogens {
+    my ($self) = @_;
+    my $h_count = $self->calc_implicit_hydrogens;
+    $self->implicit_hydrogens($h_count);
+}
+
 =item $atom->aromatic($bool)
 
 Set or get whether the atom is considered to be aromatic. This property may be
@@ -360,15 +478,7 @@ detection"! (For that, look at the L<Chemistry::Ring> module).
 
 =cut
 
-sub aromatic {
-    my $self = shift;
-    if (@_) {
-        ($self->{aromatic}) = @_;
-        return $self;
-    } else {
-        return $self->{aromatic};
-    }
-}
+Chemistry::Obj::accessor('aromatic');
 
 =item $atom->valence
 
@@ -385,9 +495,10 @@ sub valence {
     $valence;
 }
 
-=item $atom->valence
+=item $atom->explicit_valence
 
-Like c<valence>, but excluding implicit hydrogen atoms.
+Like C<valence>, but excluding implicit hydrogen atoms. To get the raw number
+of bonds, without counting bond orders, call $atom->bonds in scalar context.
 
 =cut
 
@@ -398,15 +509,12 @@ sub explicit_valence {
     $valence;
 }
 
-=item $atom->add_bond($bond)
-
-Adds a new bond to the atom, as defined by the Chemistry::Bond object $bond.
-
-=cut
-
+# this method is for internal use only; called by $mol->add_bond
 sub add_bond {
     my $self = shift;
     my $bond = shift;
+    my %seen;
+    #return if first { $_ eq $bond } @{$self->{bonds}}; 
 
     for my $atom (@{$bond->{atoms}}){ #for each atom...
         if ($atom ne $self) {
@@ -438,16 +546,22 @@ sub delete_bond {
 
 =item $atom->delete
 
-Calls $mol->delete_atom($atom) on the atom's parent molecule. Note that an atom
-should belong to only one molecule or strange things will happen.
+Calls $mol->delete_atom($atom) on the atom's parent molecule.
 
 =cut
 
 sub delete {
     my ($self) = @_;
-    $self->{parent}->delete_atom($self);
+    $self->{parent}->_delete_atom($self);
 }
 
+=item $atom->parent
+
+Returns the atom's containing object (the molecule to which the atom belongs).
+An atom can only have one parent.
+
+=cut
+
 sub parent {
     my $self = shift;
     if (@_) {
@@ -473,7 +587,7 @@ sub neighbors {
     my @ret = ();
 
     for my $b (@{$self->{bonds}}) {
-	push @ret, $b->{to} unless $from && $b->{to} eq $from;
+        push @ret, $b->{to} unless $from && $b->{to} eq $from;
     }
     @ret;
 }
@@ -491,7 +605,7 @@ sub bonds {
     my @ret = ();
 
     for my $b (@{$self->{bonds}}) {
-	push @ret, $b->{bond} unless $from && $b->{to} eq $from;
+        push @ret, $b->{bond} unless $from && $b->{to} eq $from;
     }
     @ret;
 }
@@ -515,7 +629,7 @@ sub bonds_neighbors {
     my @ret = ();
 
     for my $b (@{$self->{bonds}}) {
-	push @ret, {%$b} unless $from && $b->{to} eq $from;
+        push @ret, {%$b} unless $from && $b->{to} eq $from;
     }
     @ret;
 }
@@ -560,7 +674,8 @@ sub distance {
 
 Returns the angle in radians between the atoms involved. $atom2 is the atom in
 the middle. Can also be called as Chemistry::Atom::angle($atom1, $atom2,
-$atom3). This function can be exported.
+$atom3). This function can be exported. Note: if you override this method,
+you may also need to override angle_deg or strange things may happen.
 
 =cut
 
@@ -569,13 +684,15 @@ sub angle {
     @_ == 3 or croak "Chemistry::Atom::angle requires three atoms!\n";
     my @c;
     for my $a (@_) { # extract coordinates
+        ref $a or croak "Chemistry::Atom::angle: $a is not an object";
         push @c, $a->isa("Chemistry::Atom") ? $a->coords :
             $a->isa("Math::VectorReal") ? $a : 
                 croak "angle: $a is neither an atom nor a vector!\n";
     }
     my $v1 = $c[0] - $c[1];
     my $v2 = $c[2] - $c[1];
-    acos(($v1 . $v2) / ($v1->length * $v2->length));
+    my $l = ($v1->length * $v2->length) or return 0;
+    acos(($v1 . $v2) / $l);
 }
 
 =item $atom->angle_deg($atom2, $atom3)
@@ -592,7 +709,8 @@ sub angle_deg {
 
 Returns the dihedral angle in radians between the atoms involved.  Can also be
 called as Chemistry::Atom::dihedral($atom1, $atom2, $atom3, $atom4). May be
-exported.
+exported. Note: if you override this method, you may also need to override 
+dihedral_deg and angle or strange things may happen.
 
 =cut
 
@@ -705,7 +823,7 @@ sub sprintf {
 =item $atom->printf($format)
 
 Same as $atom->sprintf, but prints to standard output automatically. Used
-for quick and dirty molecular information dumping.
+for quick and dirty atomic information dumping.
 
 =cut
 
@@ -720,12 +838,13 @@ sub printf {
 
 =head1 VERSION
 
-0.26
+0.35
 
 =head1 SEE ALSO
 
 L<Chemistry::Mol>, L<Chemistry::Bond>, 
-L<Math::VectorReal>, L<Chemistry::Tutorial>
+L<Math::VectorReal>, L<Chemistry::Tutorial>,
+L<Chemistry::InternalCoords>
 
 The PerlMol website L<http://www.perlmol.org/>
 
@@ -735,8 +854,8 @@ Ivan Tubert-Brohman E<lt>itub@cpan.orgE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2004 Ivan Tubert-Brohman. All rights reserved. This program is free
-software; you can redistribute it and/or modify it under the same terms as
+Copyright (c) 2005 Ivan Tubert-Brohman. All rights reserved. This program is
+free software; you can redistribute it and/or modify it under the same terms as
 Perl itself.
 
 =cut
