@@ -1,6 +1,6 @@
 package Chemistry::Mol;
-$VERSION = '0.25';
-# $Id: Mol.pm,v 1.30 2004/06/30 21:58:53 itubert Exp $
+$VERSION = '0.35';
+# $Id: Mol.pm,v 1.48 2005/05/20 19:01:04 itubert Exp $
 
 =head1 NAME
 
@@ -27,8 +27,8 @@ only a minimal set of attributes is provided by default, and some attributes
 have very loosely defined meaning. This is because each program and file type
 has different idea of what each concept (such as bond and atom type) means.
 Bonds are defined as a list of atoms (typically two) with an arbitrary type.
-Atoms are defined by a symbol and a Z, and may have 3D coordinates (2D and
-internal coming soon).
+Atoms are defined by a symbol and a Z, and may have 3D and internal coordinates
+(2D coming soon).
 
 =cut
 
@@ -50,11 +50,10 @@ our %EXPORT_TAGS = (
 
 
 my %FILE_FORMATS = ();
-my $N = 0; # atom ID counter
 
 =head1 METHODS
 
-See also Chemistry::Obj for generic attributes.
+See also L<Chemistry::Obj> for generic attributes.
 
 =over 4
 
@@ -76,23 +75,19 @@ sub new {
     my $class = shift;
     my %args = @_;
     my $self = bless {
-	id => $class->nextID,
-	byId => {}, 
-	atoms => [], 
-	bonds => [], 
-	name => "",
+        id => $class->nextID,
+        byId => {}, 
+        atoms => [], 
+        bonds => [], 
+        name => "",
     }, ref $class || $class;
     $self->$_($args{$_}) for (keys %args);
     return $self;
 }
 
-sub nextID {
-    "mol".++$N; 
-}
-
-sub reset_id {
-    $N = 0; 
-}
+my $N = 0; # atom ID counter
+sub nextID { "mol".++$N; }
+sub reset_id { $N = 0; }
 
 =item $mol->add_atom($atom, ...)
 
@@ -125,8 +120,8 @@ sub add_atom_np {
 =item $mol->atom_class
 
 Returns the atom class that a molecule or molecule class expects to use by
-default. Chemistry::Mol objects return "Chemistry::Atom", but subclasses will
-likely override this method.
+default. L<Chemistry::Mol> objects return "Chemistry::Atom", but subclasses
+will likely override this method.
 
 =cut
 
@@ -136,7 +131,7 @@ sub atom_class {
 
 =item $mol->new_atom(name => value, ...)
 
-Shorthand for $mol->add_atom($mol->atom_class->new(name => value, ...));
+Shorthand for C<< $mol->add_atom($mol->atom_class->new(name => value, ...)) >>.
 
 =cut
 
@@ -147,34 +142,40 @@ sub new_atom {
 
 =item $mol->delete_atom($atom, ...)
 
-Deletes an atom from the molecule. It automatically deletes all the bonds
-in which the atom participates as well.
+Deletes an atom from the molecule. It automatically deletes all the bonds in
+which the atom participates as well. $atom should be a Chemistry::Atom
+reference. This method also accepts the atom index, but this use is deprecated
+(and buggy if multiple indices are given, unless they are in descending order).
 
 =cut
 
-# mol deletes bonds that belonged to atom
-# mol deletes atom
-
 sub delete_atom {
     my $self = shift;
-    for my $i (@_){
-        my ($atom, $index);
+    for my $i (@_) {
+        my ($atom);
         if (ref $i) {
             $atom = $i;
-            $index = $self->get_atom_index($atom)    
-                or croak "$self->delete_atom: no such atom $atom\n";
         } else {
-            $index = $i;
-            $atom = $self->atoms($index)
-                or croak "$self->delete_atom: no such atom $index\n";
+            $atom = $self->atoms($i)
+                or croak "$self->delete_atom: no such atom $i\n";
         }
-        my $id = $atom->id;
-        $self->delete_bond($atom->bonds);
-        delete $self->{byId}{$id};
-        splice @{$self->{atoms}}, $index - 1, 1;
+        $atom->delete($i);
     }
 }
 
+# takes an atom ref to delete and optionally the atom index
+# 1) deletes bonds that belonged to atom
+# 2) deletes atom
+sub _delete_atom {
+    my ($self, $atom) = @_;
+    my $index = $self->get_atom_index($atom)    
+        or croak "$self->delete_atom: no such atom $atom\n";
+    my $id = $atom->id;
+    $self->delete_bond($atom->bonds);
+    delete $self->{byId}{$id};
+    splice @{$self->{atoms}}, $index - 1, 1;
+}
+
 =item $mol->add_bond($bond, ...)
 
 Add one or more Bond objects to the molecule. Returns the last bond added.
@@ -188,7 +189,11 @@ sub add_bond {
             #croak "Duplicate ID when adding bond '$bond' to mol '$self'";
         #}
         push @{$self->{bonds}}, $bond;
-	$self->{byId}{$bond->id} = $bond;
+        $self->{byId}{$bond->id} = $bond;
+        if ($bond->{deleted}) {
+            $_->add_bond($bond) for $bond->atoms;
+            $bond->{deleted} = 0;
+        }
         $bond->parent($self);
     }
     $_[-1];
@@ -198,7 +203,7 @@ sub add_bond_np {
     my $self = shift;
     for my $bond (@_){
         push @{$self->{bonds}}, $bond;
-	$self->{byId}{$bond->id} = $bond;
+        $self->{byId}{$bond->id} = $bond;
     }
     $_[-1];
 }
@@ -206,8 +211,8 @@ sub add_bond_np {
 =item $mol->bond_class
 
 Returns the bond class that a molecule or molecule class expects to use by
-default.  Chemistry::Mol objects return "Chemistry::Bond", but subclasses will
-likely override this method.
+default. L<Chemistry::Mol> objects return "Chemistry::Bond", but subclasses
+will likely override this method.
 
 =cut
 
@@ -217,7 +222,7 @@ sub bond_class {
 
 =item $mol->new_bond(name => value, ...)
 
-Shorthand for $mol->add_bond($mol->bond_class->new(name => value, ...));
+Shorthand for C<< $mol->add_bond($mol->bond_class->new(name => value, ...)) >>.
 
 =cut
 
@@ -248,7 +253,7 @@ sub get_atom_index {
 
 =item $mol->delete_bond($bond, ...)
 
-Deletes a bond from the molecule.
+Deletes a bond from the molecule. $bond should be a L<Chemistry::Bond> object.
 
 =cut
 
@@ -258,23 +263,27 @@ Deletes a bond from the molecule.
 sub delete_bond {
     my $self = shift;
     for my $i (@_){
-        my ($bond, $index);
+        my ($bond);
         if (ref $i) {
             $bond = $i;
-            $index = $self->get_bond_index($bond)
-                or croak "$self->delete_bond: no such bond $bond\n";
         } else {
-            $index = $i;
-            $bond = $self->bonds($index)
-                or croak "$self->delete_bond: no such bond $index\n";
+            $bond = $self->bonds($i)
+                or croak "$self->delete_bond: no such bond $i\n";
         }
-        my $id = $bond->id;
-        delete $self->{byId}{$id};
-        splice @{$self->{bonds}}, $index - 1, 1;
-        $bond->delete_atoms;
+        $bond->delete;
     }
 }
 
+sub _delete_bond {
+    my ($self, $bond) = @_;
+    my $index = $self->get_bond_index($bond)    
+        or croak "$self->delete_bond: no such bond $bond\n";
+    my $id = $bond->id;
+    delete $self->{byId}{$id};
+    splice @{$self->{bonds}}, $index - 1, 1;
+    $bond->delete_atoms;
+}
+
 =item $mol->by_id($id)
 
 Return the atom or bond object with the corresponding id.
@@ -287,6 +296,13 @@ sub by_id {
     $self->{byId}{$id};
 }
 
+sub _change_id {
+    my ($self, $old_id, $new_id) = @_;
+    my $ref = $self->{byId}{$old_id};
+    $self->{byId}{$new_id} = $ref;
+    delete $self->{byId}{$old_id};
+}
+
 =item $mol->atoms($n1, ...)
 
 Returns the atoms with the given indices, or all by default. 
@@ -296,8 +312,8 @@ Indices start from one, not from zero.
 
 sub atoms {
     my $self = shift;
-    my @ats = map {$_ - 1} @_;
-    if (@ats) {
+    if (@_) {
+        my @ats = map {$_ - 1} @_;
         @{$self->{atoms}}[@ats];
     } else {
         @{$self->{atoms}};
@@ -316,8 +332,6 @@ sub atoms_by_name {
     my $re = qr/^$_[0]$/;
     no warnings;
     my @ret = grep {$_->name =~ $re} $self->atoms;
-    #my ($re) = @_; # 5.004 hack
-    #my @ret = grep {defined $_->name and $_->name =~ /$re/o} $self->atoms;
     wantarray ? @ret : $ret[0];
 }
 
@@ -325,11 +339,11 @@ sub atoms_by_name {
 
 Sort the atoms in the molecule by using the comparison function given in
 $sub_ref. This function should take two atoms as parameters and return -1, 0,
-or 1 depending on whether the first atom should go first, same, or after the
+or 1 depending on whether the first atom should go before, same, or after the
 second atom. For example, to sort by atomic number, you could use the
 following:
 
-    $mol->sort( sub { $_[0]->Z <=> $_[1]->Z } );
+    $mol->sort_atoms( sub { $_[0]->Z <=> $_[1]->Z } );
 
 Note that the atoms are passed as parameters and not as the package variables
 $a and $b like the core sort function does. This is because $mol->sort will
@@ -355,8 +369,8 @@ Indices start from one, not from zero.
 
 sub bonds {
     my $self = shift;
-    my @bonds = map {$_ - 1} @_;
-    if (@bonds) {
+    if (@_) {
+        my @bonds = map {$_ - 1} @_;
         @{$self->{bonds}}[@bonds];
     } else {
         @{$self->{bonds}};
@@ -394,7 +408,7 @@ END
     $ret;
 }
 
-=item my $info =  $mol->sprintf($format)
+=item $s = $mol->sprintf($format)
 
 Format interesting molecular information in a concise way, as specified by
 a printf-like format.
@@ -404,6 +418,7 @@ a printf-like format.
     %f{formula with format} - (note: right braces within
         the format should be escaped with a backslash)
     %s - SMILES representation
+    %S - canonical SMILES representation
     %m - mass
     %8.3m - mass, formatted as %8.3f with core sprintf
     %q - formal charge
@@ -418,6 +433,9 @@ For example, if you want just about everything:
     $mol->sprintf("%s - %n (%f). %a atoms, %b bonds; "
         . "mass=%m; charge =%q; type=%t; id=%i");
 
+Note that you have to C<use Chemistry::File::SMILES> before using C<%s> or
+C<%S> on C<< $mol->sprintf >>.
+
 =cut
 
 sub sprintf {
@@ -428,6 +446,7 @@ sub sprintf {
     $format =~ s/(?<!\\)%f\{(.*?)(?<!\\)\}/$mol->formula($1)/eg; # %f{}
     $format =~ s/(?<!\\)%f/$mol->formula/eg;                    # %f
     $format =~ s/(?<!\\)%s/$mol->print(format=>'smiles')/eg;    # %s
+    $format =~ s/(?<!\\)%S/$mol->print(format=>'smiles', unique => 1)/eg;    # %s
     $format =~ s/(?<!\\)%n/$mol->name/eg;                       # %n
     $format =~ s/(?<!\\)%(\d*\.?\d*)m/
         $1 ? sprintf "%$1f", $mol->mass : $mol->mass/eg;        # %m
@@ -442,8 +461,8 @@ sub sprintf {
 
 =item $mol->printf($format)
 
-Same as $mol->sprintf, but prints to standard output automatically. Used
-for quick and dirty molecular information dumping.
+Same as C<< $mol->sprintf >>, but prints to standard output automatically.
+Used for quick and dirty molecular information dumping.
 
 =cut
 
@@ -454,7 +473,7 @@ sub printf {
 
 =item Chemistry::Mol->parse($string, option => value...)
 
-Parse the molecule encoded in $string. The format should be specified
+Parse the molecule encoded in C<$string>. The format should be specified
 with the the C<format> option; otherwise, it will be guessed.
 
 =cut
@@ -470,18 +489,23 @@ sub parse {
         croak "Parse does not support autodetection yet.",
             "Please specify a format.";
     }
-    undef;
+    return;
 }
 
 =item Chemistry::Mol->read($fname, option => value ...)
 
-Read a file and return a list of Mol objects, or croaks if there
-was a problem. The type of file will be guessed if not
-specified via the C<format> option.
+Read a file and return a list of Mol objects, or croaks if there was a problem.
+The type of file will be guessed if not specified via the C<format> option.
 
-Note that only registered file readers will be used. Readers may
-be registered using register_type(); modules that include readers
-(such as Chemistry::File::PDB) usually register them automatically.
+Note that only registered file readers will be used. Readers may be registered
+using C<register_type()>; modules that include readers (such as
+L<Chemistry::File::PDB>) usually register them automatically when they are
+loaded.
+
+Automatic decompression of gzipped files is supported if the L<Compress::Zlib>
+module is installed. Files ending in .gz are assumed to be compressed;
+otherwise it is possible to force decompression by passing the gzip => 1
+option (or no decompression with gzip => 0).
 
 =cut
 
@@ -509,12 +533,17 @@ sub read {
 
 =item $mol->write($fname, option => value ...)
 
-Write a molecule file, or croak if there
-was a problem. The type of file will be guessed if not
-specified via the C<format> option.
+Write a molecule file, or croak if there was a problem. The type of file will
+be guessed if not specified via the C<format> option.
 
 Note that only registered file formats will be used. 
 
+Automatic gzip compression is supported if the IO::Zlib module is installed.
+Files ending in .gz are assumed to be compressed; otherwise it is possible to
+force compression by passing the gzip => 1 option (or no compression with gzip
+=> 0). Specific compression levels between 2 (fastest) and 9 (most compressed)
+may also be used (e.g., gzip => 9).
+
 =cut
 
 sub write {
@@ -523,24 +552,72 @@ sub write {
     if ($opts{format}) {
         return $self->formats($opts{format})->write_file(@_);
     } else { # guess format
-	for my $type ($self->formats) {
+        for my $type ($self->formats) {
             if ($self->formats($type)->name_is($fname)) {
                 return $self->formats($type)->write_file(@_);
-	    }
-	}
+            }
+        }
     }
     croak "Couldn't guess format for writing file '$fname'";
 }
 
+=item Chemistry::Mol->file($file, option => value ...)
+
+Create a L<Chemistry::File>-derived object for reading or writing to a file.
+The object can then be used to read the molecules or other information in the
+file.
+
+This has more flexibility than calling C<< Chemistry::Mol->read >> when
+dealing with multi-molecule files or files that have higher structure or that
+have information that does not belong to the molecules themselves. For
+example, a reaction file may have a list of molecules, but also general
+information like the reaction name, yield, etc. as well as the classification
+of the molecules as reactants or products. The exact information that is
+available will depend on the file reader class that is being used. The
+following is a hypothetical example for reading MDL rxnfiles.
+
+    # assuming this module existed...
+    use Chemistry::File::Rxn;
+
+    my $rxn = Chemistry::Mol->file('test.rxn');
+    $rxn->read;
+    $name      = $rxn->name;
+    @reactants = $rxn->reactants; # mol objects
+    @products  = $rxn->products;
+    $yield     = $rxn->yield;     # a number
+
+Note that only registered file readers will be used. Readers may be registered
+using register_type(); modules that include readers (such as
+Chemistry::File::PDB) usually register them automatically.
+
+=cut
+
+sub file {
+    my ($self,  $file, %opts) = @_;
+    %opts = (mol_class => $self, %opts);
+
+    if ($opts{format}) {
+        return $self->formats($opts{format})->new(file => $file, 
+            opts => \%opts);
+    } else { # guess format
+        for my $type ($self->formats) {
+            if ($self->formats($type)->file_is($file)) {
+                return $self->formats($type)->new(file => $file, 
+                    opts => \%opts);
+            }
+        }
+    }
+    croak "Couldn't guess format of file '$file'";
+}
+
 =item Chemistry::Mol->register_format($name, $ref)
 
-Register a file type. The identifier $name must be unique.
-$ref is either a class name (a package) or an object that complies
-with the L<Chemistry::File> interface (e.g., a subclass of Chemistry::File).
-If $ref is omitted, the calling package is used automatically. More than one
-format can be registered at a time, but then $ref must be included for each
-format (e.g., Chemistry::Mol->register_format(format1 => "package1", format2 =>
-package2).
+Register a file type. The identifier $name must be unique.  $ref is either a
+class name (a package) or an object that complies with the L<Chemistry::File>
+interface (e.g., a subclass of Chemistry::File).  If $ref is omitted, the
+calling package is used automatically. More than one format can be registered
+at a time, but then $ref must be included for each format (e.g.,
+Chemistry::Mol->register_format(format1 => "package1", format2 => package2).
 
 The typical user doesn't have to care about this function. It is used
 automatically by molecule file I/O modules.
@@ -580,7 +657,8 @@ sub formats {
 
 =item $mol->mass
 
-Return the molar mass.
+Return the molar mass. This is just the sum of the masses of the atoms.  See
+L<Chemistry::Atom>::mass for details such as the handling of isotopes.
 
 =cut
 
@@ -596,7 +674,8 @@ sub mass {
 =item $mol->charge
 
 Return the charge of the molecule. By default it returns the sum of the formal
-charges of the atoms.
+charges of the atoms. However, it is possible to set an arbitrary charge by
+calling C<< $mol->charge($new_charge) >>
 
 =cut
 
@@ -646,16 +725,45 @@ sub formula {
 
 =item my $mol2 = $mol->clone;
 
-Makes a copy of a molecule.
+Makes a copy of a molecule. Note that this is a B<deep> copy; if your molecule
+has a pointer to the rest of the universe, the entire universe will be cloned!
 
 =cut
 
 sub clone {
     my ($self) = @_;
     my $clone = dclone $self;
-    $clone->_weaken;
+    $clone->_weaken if Storable->VERSION < 2.14;
+    $clone;
 }
 
+=item my $mol2 = $mol->safe_clone;
+
+Like clone, it makes a deep copy of a molecule. The difference is that the copy
+is not "exact" in that new molecule and its atoms and bonds get assigned new
+IDs. This makes it safe to combine cloned molecules. For example, this is an
+error:
+
+    # XXX don't try this at home!
+    my $mol2 = Chemistry::Mol->combine($mol1, $mol1);
+    # the atoms in $mol1 will clash
+
+But this is ok:
+
+    # the "safe clone" of $mol1 will have new IDs
+    my $mol2 = Chemistry::Mol->combine($mol1, $mol1->safe_clone);
+
+=cut
+
+sub safe_clone {
+    my ($mol) = @_;
+    my $clone = $mol->clone;
+    for ($clone, $clone->atoms, $clone->bonds) {
+        $_->id($_->nextID);
+    }
+    $clone;
+} 
+
 sub _weaken {
     my ($self) = @_;
     for ($self->atoms, $self->bonds) {
@@ -663,6 +771,7 @@ sub _weaken {
     }
     $self;
 }
+
 =item ($distance, $atom_here, $atom_there) = $mol->distance($obj)
 
 Returns the minimum distance to $obj, which can be an atom, a molecule, or a
@@ -677,7 +786,7 @@ sub distance {
     my ($self, $other) = @_;
     if ($other->isa("Chemistry::Mol")) {
         my @atoms = $self->atoms;
-        my $atom = shift @atoms or return undef; # need at least one atom
+        my $atom = shift @atoms or return; # need at least one atom
         my $closest_here = $atom;
         my ($min_length, $closest_there) = $atom->distance($other);
         for $atom (@atoms) {
@@ -703,12 +812,12 @@ Combines several molecules in one bigger molecule. If called as a class method,
 as in the first example, it returns a new combined molecule without altering
 any of the parameters. If called as an instance method, as in the second
 example, all molecules are combined into $mol1 (but $mol2, $mol3, ...) are not
-altered.
+altered. B<Note>: Make sure you don't combine molecules which contain atoms
+with duplicate IDs (for example, if they were cloned).
 
 =cut
 
 # joins several molecules into one
-# Does not touch the original copy.
 sub combine {
     my ($self, @others) = @_;
     my $mol;
@@ -761,23 +870,65 @@ sub separate {
     @mols;
 }
 
+# this method fills the _paint_tab attribute for every atom connected
+# to the given start atom $atom with $color. Used for separating
+# connected fragments. Uses a depth-first search
 sub _paint {
     my ($self, $atom, $color) = @_;
-    return if $self->{_paint_tab}{$atom->id} eq $color;
+    return if defined $self->{_paint_tab}{$atom->id};
     $self->{_paint_tab}{$atom->id} = $color;
-    $self->{_paint_tab}{$_->id} = $color for ($atom->bonds);
+    $self->{_paint_tab}{$_->id}    = $color for ($atom->bonds);
     for my $neighbor ($atom->neighbors) {
         $self->_paint($neighbor, $color);
     }
 }
 
+=item $mol->sprout_hydrogens
+
+Convert all the implicit hydrogen atoms in the molecule to explicit atoms.
+It does B<not> generate coordinates for the atoms.
+
+=cut
+
+sub sprout_hydrogens {
+    my ($self) = @_;
+    $_->sprout_hydrogens for $self->atoms;
+}
+
+=item $mol->collapse_hydrogens
+
+Convert all the explicit hydrogen atoms in the molecule to implicit hydrogens.
+(Exception: hydrogen atoms that are adjacent to a hydrogen atom are not
+collapsed.)
+
+=cut
+
+sub collapse_hydrogens {
+    my ($self) = @_;
+    for my $atom (grep { $_->symbol ne 'H' } $self->atoms) {
+        $atom->collapse_hydrogens;
+    }
+}
+
+=item $mol->add_implicit_hydrogens
+
+Use heuristics to figure out how many implicit hydrogens should each atom in 
+the molecule have to satisfy its normal "organic" valence.
+
+=cut
+
+sub add_implicit_hydrogens {
+    my ($self) = @_;
+    $_->add_implicit_hydrogens for $self->atoms;
+}
+
 1;
 
 =back
 
 =head1 VERSION
 
-0.25
+0.35
 
 =head1 SEE ALSO
 
@@ -788,12 +939,12 @@ The PerlMol website L<http://www.perlmol.org/>
 
 =head1 AUTHOR
 
-Ivan Tubert E<lt>itub@cpan.orgE<gt>
+Ivan Tubert-Brohman E<lt>itub@cpan.orgE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2004 Ivan Tubert. All rights reserved. This program is free
-software; you can redistribute it and/or modify it under the same terms as
+Copyright (c) 2005 Ivan Tubert-Brohman. All rights reserved. This program is
+free software; you can redistribute it and/or modify it under the same terms as
 Perl itself.
 
 =cut
